/*
   Sequential search for "aardvark" in each file in the current directory.

   This version does its own buffering of I/O from a FileInputStream.

   This version is incorrect in that it will miss any "aardvark"
   matches that span across two buffers (which is not very likely
   to happen).
*/

import java.time.Instant;
import java.time.Duration;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.io.IOException;
import java.io.RandomAccessFile;
//import java.nio.MappedByteBuffer;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.nio.file.Path;

public class SearchSequentialMemoryMappedFiles
{
   private static final int BUFFERSIZE = 16 * 8192;
   private static int totalLines = 0;
   private static final SimpleDateFormat ft = new SimpleDateFormat("hh:mm:ss");

   public static void main(String[] args) throws IOException
   {
      System.out.println(ft.format(new Date()) + " Search Sequential in all Memory Mapped Files");
      Instant start = Instant.now();

      Files.list(Paths.get("."))
             .filter(p -> p.toString().endsWith(".txt"))
               .forEach(p -> searchInFile(p));

      Instant stop = Instant.now();
      long time = Duration.between(start, stop).toMillis();
      System.out.printf("%s processed %,d lines from all files.\n",
                       ft.format(new Date()), totalLines);
      System.out.printf("elapsed time: %,dms\n", time);
   }


   public static void searchInFile(Path path)
   {
      int lineNumber = 1;

      try
      {
         String fileName = path.getFileName().toString();
         FileChannel fc = new RandomAccessFile(fileName, "r").getChannel();
         long fileSize = fc.size(); // assume that each file is < 2GB

         ByteBuffer mbb = fc.map(FileChannel.MapMode.READ_ONLY,
                                 0,
                                 fileSize );

         // Filter the data in the buffer.
         for (int i = 0; i < fileSize; ++i)
         {
            if ( '\n' == mbb.get(i) )
            {
               ++lineNumber;
            }
            else if ( i < fileSize - 8
                   && 'a' == mbb.get(i)
                   && 'a' == mbb.get(i+1)
                   && 'r' == mbb.get(i+2)
                   && 'd' == mbb.get(i+3)
                   && 'v' == mbb.get(i+4)
                   && 'a' == mbb.get(i+5)
                   && 'r' == mbb.get(i+6)
                   && 'k' == mbb.get(i+7) )
            {
               System.out.printf("%s line %,d in %s (current total line no %,d).\n",
                                  ft.format(new Date()),
                                  lineNumber,
                                  fileName,
                                  lineNumber + totalLines);
            }
         }
         totalLines += lineNumber;
      }
      catch(IOException e){throw new Error(e);}
   }
}
