/*
   Sequential search for "aardvark" in each file in the current directory.

   This version does its own buffering of I/O from a FileInputStream.

   This version is incorrect in that it will miss any "aardvark"
   matches that span across two buffers (which is not very likely
   to happen).
*/

import java.time.Instant;
import java.time.Duration;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.io.IOException;
import java.io.InputStream;
import java.io.FileInputStream;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.nio.file.Path;

public class SearchSequential_v4_Files
{
   private static final int BUFFERSIZE = 16 * 8192;
   private static int totalLines = 0;
   private static final SimpleDateFormat ft = new SimpleDateFormat("hh:mm:ss");

   public static void main(String[] args) throws IOException
   {
      System.out.println(ft.format(new Date()) + " Search Sequential ver 4 in all Files");
      Instant start = Instant.now();

      Files.list(Paths.get("."))
             .filter(p -> p.toString().endsWith(".txt"))
               .forEach(p -> searchInFile(p));

      Instant stop = Instant.now();
      long time = Duration.between(start, stop).toMillis();
      System.out.printf("%s processed %,d lines from all files.\n",
                       ft.format(new Date()), totalLines);
      System.out.printf("elapsed time: %,dms\n", time);
   }


   public static void searchInFile(Path path)
   {
      byte[] c = new byte[BUFFERSIZE];
      int n;
      int lineNumber = 1;

      try
      {
         final String fileName = path.getFileName().toString();
         final InputStream is = new FileInputStream( fileName );

         while ( (n = is.read(c)) != -1 )
         {
            for (int i = 0; i < n; ++i)
            {
               if ( '\n' == c[i] )
               {
                  ++lineNumber;
               }
               else if ( i < n - 8
                      && 'a' == c[i]
                      && 'a' == c[i+1]
                      && 'r' == c[i+2]
                      && 'd' == c[i+3]
                      && 'v' == c[i+4]
                      && 'a' == c[i+5]
                      && 'r' == c[i+6]
                      && 'k' == c[i+7] )
               {
                  System.out.printf("%s line %,d in %s (total line no %,d).\n",
                                     ft.format(new Date()),
                                     lineNumber,
                                     fileName,
                                     lineNumber + totalLines);
               }
            }
         }
         is.close();
         totalLines += lineNumber;
      }
      catch(IOException e){throw new Error(e);}
   }
}
